#US county-level agricultural crop production typology building
#Script 1/2
#Description: cluster analysis on county level crop acreage data
#Input (mydata) is output of script 1 (1_Crop_cluster_data_wrangling_forRepository_071919.R)
#Author: Courtney Hammond Wagner
#email: courtney.hammond@uvm.edu
#Date revised: 5/15/2019

library(dplyr)
library(clValid)


mydata <- read.csv("22crop_percent_data_to_cluster_final.csv")
names(mydata)
mydata <- mydata[,-1]
mydata <- as.data.frame(mydata)
mydata$ANSI<- formatC(mydata$ANSI, width = 5, format = "d", flag = "0")

#Note ANSI 08025, 32021, 34031, and 36047 are NAs in 22 crop percent data because there are 
  #NA's for total county cropland, so the 22nd crop variable couldn't be created 
    #(it involved subtracting from an NA...)
#Drop these four counties
mydata <- mydata[complete.cases(mydata),]


####################
#cluster analysis
#######################

#Percent composition
#####
#descriptive statistics of percent acreage data
desc_stats_per <- data.frame(
  Min = apply(mydata[2:23], 2, min, na.rm = TRUE), # minimum
  Med = apply(mydata[2:23], 2, median, na.rm = TRUE), # median
  Mean = apply(mydata[2:23], 2, mean, na.rm = TRUE), # mean
  SD = apply(mydata[2:23], 2, sd, na.rm = TRUE), # Standard deviation
  Max = apply(mydata[2:23], 2, max, na.rm = TRUE) # Maximum
)

#K-means on percent acreage

mydata.scale <- scale(mydata[2:23]) # scale variables
#Run analysis on non-scaled data?

#descriptive statistics of scaled data mean 0 sd 1
desc_stats_scale <- data.frame(
  Min = apply(mydata.scale, 2, min, na.rm = TRUE), # minimum
  Med = apply(mydata.scale, 2, median, na.rm = TRUE), # median
  Mean = apply(mydata.scale, 2, mean, na.rm = TRUE), # mean
  SD = apply(mydata.scale, 2, sd, na.rm = TRUE), # Standard deviation
  Max = apply(mydata.scale, 2, max, na.rm = TRUE) # Maximum
)

# Determine number of clusters
#1)Look for a bend or elbow in the sum of squared error (SSE) scree plot
wss <- (nrow(mydata.scale)-1)*sum(apply(mydata.scale,2,var))
for (i in 1:20) wss[i] <- sum(kmeans(mydata.scale, 
                                     centers=i)$withinss)
plot(1:20, wss, type="b", xlab="Number of Clusters",
     ylab="Within groups sum of squares") 
#look for bend in plot to determine num of clusters to extract



#Cluster Validtion package - can compare validation indices using different clustering
  #algorithms and number of clusters
#internal validity
intern <- clValid(mydata.scale, 6:12, clMethods = c("kmeans"), validation = "internal")
summary(intern)

op <- par(no.readonly = TRUE)
par(mfrow = c(2, 2), mar = c(4, 4, 3, 1))
plot(intern, legend = FALSE)


#stability
stab <- clValid(mydata.scale, 5:12, clMethods = "kmeans", validation = "stability")
optimalScores(stab)


#10 clusters
set.seed(241)
fit10 <- kmeans(mydata.scale, 10, nstart = 25) # cluster solution
#get cluster means (per cluster)
cluster10.means <- aggregate(mydata.scale,by=list(fit10$cluster),FUN=mean)
#write.csv(cluster10.means, "cluster10means_22crop.csv")
fit10$size
cluster10_22crop <- fit10$cluster
length(cluster10_22crop)



mydata_with_clusters <- cbind(mydata,fit10$cluster)


####Save Output
write.csv(mydata_with_clusters, "wide_data_with_clusters_final.csv")



